#!/usr/bin/perl
# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package psp;
our $VERSION='0.7';

use warnings;
use strict;
use PSP::Engine;
use PSP::Config;
use PSP::CGI::HTTP;
use Cwd;

my $configFile='./psp.conf';

sub main{
	my $file=defined($ENV{PATH_INFO})?($ENV{PATH_INFO}):('');
	my $conf=new PSP::Config();
	my $psp=new PSP::Engine(conf => $conf);
	$conf->addFile($configFile);
	# Require leading slash in filename if filename given
	if(defined($conf->{ExtraConfig}) && -f $conf->{ExtraConfig}){
		$conf->addFile($conf->{ExtraConfig});
	}
	unless($file eq '' || $file=~/^\//){
		$file='/'.$file ;
	}
	if(defined(filenameIsSecure($conf,$file))){
		if(-d $conf->{BaseDirectory}.$file){
			if($file=~/\/$/){
				$file.=$conf->{IndexFile};
			}
			else{
				PSP::CGI::HTTP::redirect($psp,$conf->{BaseURI}.$file.'/');
				flushAndExit($psp);
			}
		}
		if(-f $conf->{BaseDirectory}.$file){
			unshift(@INC,cwd,@{$conf->{LibDirectory}});
			$psp->file($file);
			flushAndExit($psp);
		}
		else{
			PSP::CGI::HTTP::notFound($psp,$conf->{BaseURI}.$file);
			flushAndExit($psp);
		}
	}
	else{
		PSP::CGI::HTTP::status($psp,500,'Not Executed','Execution prevented due to security reasons.');
		flushAndExit($psp);
	}
}

sub flushAndExit{
	my $psp=shift;
	$psp->cgi->setheader('X-Engine' => 'PSP/'.$VERSION);
	print($psp->cgi->flushHeaders());
	print($psp->flush());
	exit(0);
}
sub filenameIsSecure{
	my $conf=shift;
	my $file=shift;
	return(undef) if(not defined($file) || $file=~/[^$conf->{FilenameCharacters}]/ || ($conf->{RequireSecureDirectory} && $file=~/\/\.\.\//));
	return($file);
}

main();
1;